﻿using System;
using UnityEngine;
using UnityEngine.UI;
using System.Collections;
using System.Collections.Generic;
using MVR.FileManagementSecure;
using SimpleJSON;
using MeshVR;
using Request = MeshVR.AssetLoader.AssetBundleFromFileRequest;
using AssetBundles;
using MVR;

namespace ConditionalTrigger
{
    public class TC_ControllerAction : TriggerCondition
    {
        protected JSONStorableStringChooser js_Key;
        protected JSONStorableStringChooser js_ActionType;
        protected JSONStorableFloat js_RefreshDelay;

        protected float lastFrame_DpadUpDown, lastFrame_DpadLeftRight, lastFrame_Triggers;

        string currentKeyName;

        bool triggered = false;
        float triggerTime = 0;

        public TC_ControllerAction(ConditionalTrigger _manager, int _id) : base(_manager, _id)
        {
            js_Key = new JSONStorableStringChooser(id + "Button", GetKeybindNames(), "[None]", "Button", Callback_keySet);
            js_ActionType = new JSONStorableStringChooser(id + "Action Type", new List<string> { "BtnDown", "BtnUp", "BtnPressed" }, "BtnDown", "Action");

            js_RefreshDelay = new JSONStorableFloat(id + "Refresh Delay", 0.03f, 0f, 2f);

            manager.RegisterFloat(js_RefreshDelay);
            manager.RegisterStringChooser(js_Key);
            manager.RegisterStringChooser(js_ActionType);
        }

        public override void Init()
        {
            
        }

        public override bool Result()
        {
            return triggered;
        }

        public override void Update()
        {
            UpdateInputs();

            lastFrame_DpadLeftRight = Input.GetAxis(JoystickControl.singleton.x360DPadXAxisName);
            lastFrame_DpadUpDown = Input.GetAxis(JoystickControl.singleton.x360DPadYAxisName);
            lastFrame_Triggers = Input.GetAxis(JoystickControl.singleton.triggersAxisName);
        }

        protected void UpdateInputs()
        {
            if (triggered)
            {
                if (Time.time - triggerTime > js_RefreshDelay.val)
                    triggered = false;

                return;
            }


            if (currentKeyName == "[None]")
                return;



            switch (currentKeyName)
            {
                case "Dpad Up":
                    switch (js_ActionType.val)
                    {
                        case "BtnDown":
                            if (Input.GetAxis(JoystickControl.singleton.x360DPadYAxisName) > 0 && lastFrame_DpadUpDown == 0)
                                triggered = true;
                            break;

                        case "BtnUp":
                            if (Input.GetAxis(JoystickControl.singleton.x360DPadYAxisName) == 0 && lastFrame_DpadUpDown > 0)
                                triggered = true;
                            break;

                        case "BtnPressed":
                            if (Input.GetAxis(JoystickControl.singleton.x360DPadYAxisName) > 0)
                                triggered = true;
                            break;
                    }
                    break;

                case "Dpad Down":
                    switch (js_ActionType.val)
                    {
                        case "BtnDown":
                            if (Input.GetAxis(JoystickControl.singleton.x360DPadYAxisName) < 0 && lastFrame_DpadUpDown == 0)
                                triggered = true;
                            break;

                        case "BtnUp":
                            if (Input.GetAxis(JoystickControl.singleton.x360DPadYAxisName) == 0 && lastFrame_DpadUpDown < 0)
                                triggered = true;
                            break;

                        case "BtnPressed":
                            if (Input.GetAxis(JoystickControl.singleton.x360DPadYAxisName) < 0)
                                triggered = true;
                            break;
                    }
                    break;

                case "Dpad Left":
                    switch (js_ActionType.val)
                    {
                        case "BtnDown":
                            if (Input.GetAxis(JoystickControl.singleton.x360DPadXAxisName) < 0 && lastFrame_DpadLeftRight == 0)
                                triggered = true;
                            break;

                        case "BtnUp":
                            if (Input.GetAxis(JoystickControl.singleton.x360DPadXAxisName) == 0 && lastFrame_DpadLeftRight < 0)
                                triggered = true;
                            break;

                        case "BtnPressed":
                            if (Input.GetAxis(JoystickControl.singleton.x360DPadXAxisName) < 0)
                                triggered = true;
                            break;
                    }
                    break;

                case "Dpad Right":
                    switch (js_ActionType.val)
                    {
                        case "BtnDown":
                            if (Input.GetAxis(JoystickControl.singleton.x360DPadXAxisName) > 0 && lastFrame_DpadLeftRight == 0)
                                triggered = true;
                            break;

                        case "BtnUp":
                            if (Input.GetAxis(JoystickControl.singleton.x360DPadXAxisName) == 0 && lastFrame_DpadLeftRight > 0)
                                triggered = true;
                            break;

                        case "BtnPressed":
                            if (Input.GetAxis(JoystickControl.singleton.x360DPadXAxisName) > 0)
                                triggered = true;
                            break;
                    }
                    break;

                case "Button Top":
                    switch (js_ActionType.val)
                    {
                        case "BtnDown":
                            if (Input.GetKeyDown(KeyCode.Joystick1Button3))
                                triggered = true;
                            break;

                        case "BtnUp":
                            if (Input.GetKeyUp(KeyCode.Joystick1Button3))
                                triggered = true;
                            break;

                        case "BtnPressed":
                            if (Input.GetKey(KeyCode.Joystick1Button3))
                                triggered = true;
                            break;
                    }
                    break;

                case "Button Bot":
                    switch (js_ActionType.val)
                    {
                        case "BtnDown":
                            if (Input.GetKeyDown(KeyCode.Joystick1Button0))
                                triggered = true;
                            break;

                        case "BtnUp":
                            if (Input.GetKeyUp(KeyCode.Joystick1Button0))
                                triggered = true;
                            break;

                        case "BtnPressed":
                            if (Input.GetKey(KeyCode.Joystick1Button0))
                                triggered = true;
                            break;
                    }
                    break;

                case "Button Left":
                    switch (js_ActionType.val)
                    {
                        case "BtnDown":
                            if (Input.GetKeyDown(KeyCode.Joystick1Button2))
                                triggered = true;
                            break;

                        case "BtnUp":
                            if (Input.GetKeyUp(KeyCode.Joystick1Button2))
                                triggered = true;
                            break;

                        case "BtnPressed":
                            if (Input.GetKey(KeyCode.Joystick1Button2))
                                triggered = true;
                            break;
                    }
                    break;

                case "Button Right":
                    switch (js_ActionType.val)
                    {
                        case "BtnDown":
                            if (Input.GetKeyDown(KeyCode.Joystick1Button1))
                                triggered = true;
                            break;

                        case "BtnUp":
                            if (Input.GetKeyUp(KeyCode.Joystick1Button1))
                                triggered = true;
                            break;

                        case "BtnPressed":
                            if (Input.GetKey(KeyCode.Joystick1Button1))
                                triggered = true;
                            break;
                    }
                    break;

                case "L1":
                    switch (js_ActionType.val)
                    {
                        case "BtnDown":
                            if (Input.GetKeyDown(KeyCode.Joystick1Button4))
                                triggered = true;
                            break;

                        case "BtnUp":
                            if (Input.GetKeyUp(KeyCode.Joystick1Button4))
                                triggered = true;
                            break;

                        case "BtnPressed":
                            if (Input.GetKey(KeyCode.Joystick1Button4))
                                triggered = true;
                            break;
                    }
                    break;

                case "L2":
                    if (Input.GetAxis(JoystickControl.singleton.triggersAxisName) < -0.5f)
                        triggered = true;
                    break;

                case "L3":
                    switch (js_ActionType.val)
                    {
                        case "BtnDown":
                            if (Input.GetKeyDown(KeyCode.Joystick1Button8))
                                triggered = true;
                            break;

                        case "BtnUp":
                            if (Input.GetKeyUp(KeyCode.Joystick1Button8))
                                triggered = true;
                            break;

                        case "BtnPressed":
                            if (Input.GetKey(KeyCode.Joystick1Button8))
                                triggered = true;
                            break;
                    }
                    break;

                case "R1":
                    switch (js_ActionType.val)
                    {
                        case "BtnDown":
                            if (Input.GetKeyDown(KeyCode.Joystick1Button5))
                                triggered = true;
                            break;

                        case "BtnUp":
                            if (Input.GetKeyUp(KeyCode.Joystick1Button5))
                                triggered = true;
                            break;

                        case "BtnPressed":
                            if (Input.GetKey(KeyCode.Joystick1Button5))
                                triggered = true;
                            break;
                    }
                    break;

                case "R2":
                    if (Input.GetAxis(JoystickControl.singleton.triggersAxisName) > 0.5f)
                        triggered = true;
                    break;

                case "R3":
                    switch (js_ActionType.val)
                    {
                        case "BtnDown":
                            if (Input.GetKeyDown(KeyCode.Joystick1Button9))
                                triggered = true;
                            break;

                        case "BtnUp":
                            if (Input.GetKeyUp(KeyCode.Joystick1Button9))
                                triggered = true;
                            break;

                        case "BtnPressed":
                            if (Input.GetKey(KeyCode.Joystick1Button9))
                                triggered = true;
                            break;
                    }
                    break;
            }

            if (triggered)
            {
                triggerTime = Time.time;
            }
        }

        public override void ShowUI()
        {
            manager.CreateTempSpacer(25f);

            manager.CreateTempStaticText("Title", "<color=#000><size=30><b>Controller Action</b></size></color>", 40);

            manager.CreateTempFilterablePopup(js_Key);
            manager.CreateTempPopup(js_ActionType);

            manager.CreateTempSlider(js_RefreshDelay);
        }

        public override void OnDestroy()
        {
            base.OnDestroy();

            manager.DeregisterStringChooser(js_Key);
            manager.DeregisterStringChooser(js_ActionType);
            manager.DeregisterFloat(js_RefreshDelay);
        }

        public override void UpdateID(int _newID)
        {
            base.UpdateID(_newID);

            js_Key.name = id + "Button";
            js_ActionType.name = id + "Action Type";
            js_RefreshDelay.name = id + "Refresh Delay";
        }

        public override bool CanReset
        {
            get
            {
                return !triggered;
            }
        }

        protected void Callback_keySet(string _newVal)
        {
            currentKeyName = _newVal;
        }


        protected List<string> GetKeybindNames()
        {
            List<string> list = new List<string>();

            list.Add("[None]");

            list.Add("Dpad Up");
            list.Add("Dpad Down");
            list.Add("Dpad Left");
            list.Add("Dpad Right");

            list.Add("Button Top");
            list.Add("Button Bot");
            list.Add("Button Left");
            list.Add("Button Right");

            list.Add("L1");
            list.Add("L2");
            list.Add("L3");
            list.Add("R1");
            list.Add("R2");
            list.Add("R3");


            return list;
        }
    }

}